# -*- coding: utf-8 -*-

import re
import sys
from .base import AipBase
from .base import base64
from .base import json
from .base import urlencode
from .base import quote

class AipImageCensor(AipBase):
    """
        Aip ImageCensor
    """

    __antiPornUrl = 'https://aip.baidubce.com/rest/2.0/antiporn/v1/detect'

    __antiPornGifUrl = 'https://aip.baidubce.com/rest/2.0/antiporn/v1/detect_gif'

    __antiTerrorUrl = 'https://aip.baidubce.com/rest/2.0/antiterror/v1/detect'

    __faceAuditUrl = 'https://aip.baidubce.com/rest/2.0/solution/v1/face_audit'
    
    __imageCensorCombUrl = 'https://aip.baidubce.com/api/v1/solution/direct/img_censor'

    __imageCensorUserDefinedUrl = 'https://aip.baidubce.com/rest/2.0/solution/v1/img_censor/user_defined'

    __antiSpamUrl = 'https://aip.baidubce.com/rest/2.0/antispam/v2/spam'
    
    def antiPorn(self, image):
        """
            antiporn
        """

        data = {}
        data['image'] = base64.b64encode(image).decode()

        return self._request(self.__antiPornUrl, data)

    def antiPornGif(self, image):
        """
            antiporn gif
        """

        data = {}
        data['image'] = base64.b64encode(image).decode()

        return self._request(self.__antiPornGifUrl, data)

    def antiTerror(self, image):
        """
            antiterror
        """

        data = {}
        data['image'] = base64.b64encode(image).decode()

        return self._request(self.__antiTerrorUrl, data)

    def faceAudit(self, images, configId=''):
        """
            faceAudit
        """

        # 非数组则处理为数组
        if not isinstance(images, list):
            images = [images]

        data = {
            'configId': configId
        }

        isUrl = images[0][0:4] == 'http'
        if not isUrl:
            data['images'] = ','.join([
                base64.b64encode(image).decode() for image in images
            ])
        else:            
            data['imgUrls'] = ','.join([
                quote(url) for url in images
            ])

        return self._request(self.__faceAuditUrl, data)

    def imageCensorComb(self, image, scenes='antiporn', options=None):
        """
            imageCensorComb
        """

        options = options or {}

        if not isinstance(scenes, list):
            scenes = scenes.split(',')
        
        data = {
            'scenes': scenes,
        }

        isUrl = image.strip()[0:4] == 'http'
        if not isUrl:
            data['image'] = base64.b64encode(image).decode()
        else:
            data['imgUrl'] = image

        data.update(options)

        return self._request(self.__imageCensorCombUrl, json.dumps(data), {
            'Content-Type': 'application/json',
        })

    def imageCensorUserDefined(self, image):
        """
            imageCensorUserDefined
        """
        
        data = {}

        isUrl = image[0:4] == 'http'
        if not isUrl:
            data['image'] = base64.b64encode(image).decode()
        else:
            data['imgUrl'] = image

        return self._request(self.__imageCensorUserDefinedUrl, data)

    def antiSpam(self, content, options=None):
        """
            anti spam
        """

        data = {}
        data['content'] = content

        return self._request(self.__antiSpamUrl, data)



# -*- coding: utf-8 -*-
"""
    aip public
"""

from .ocr import AipOcr
from .nlp import AipNlp
from .face import AipFace
from .imagecensor import AipImageCensor
from .imagecensor import AipImageCensor as AipContentCensor
from .kg import AipKg
from .speech import AipSpeech
from .imageclassify import AipImageClassify
from .imagesearch import AipImageSearch
from .bodyanalysis import AipBodyAnalysis
from .easydl import EasyDL


# -*- coding: utf-8 -*-

"""
    EasyDL
"""

from .base import AipBase
from .base import base64
from .base import hashlib
from .base import json

class EasyDL(AipBase):
    """
        EasyDL
    """

    def _isPermission(self, authObj):
        """
            check whether permission
        """

        return True

    def predictImage(self, url, image, options=None):
        """
            图像
        """

        data = {}

        data['image'] = base64.b64encode(image).decode()

        data.update(options or {})

        return self._request(url, json.dumps(data, ensure_ascii=False), {
            'Content-Type': 'application/json',
        })

    def predictSound(self, url, sound, options=None):
        """
            声音
        """

        data = {}

        data['sound'] = base64.b64encode(sound).decode()

        data.update(options or {})

        return self._request(url, json.dumps(data, ensure_ascii=False), {
            'Content-Type': 'application/json',
        })



# -*- coding: utf-8 -*-

"""
人体分析
"""

import re
import sys
import math
import time
from .base import AipBase
from .base import base64
from .base import json
from .base import urlencode
from .base import quote

class AipBodyAnalysis(AipBase):

    """
    人体分析
    """

    __bodyAnalysisUrl = 'https://aip.baidubce.com/rest/2.0/image-classify/v1/body_analysis'

    __bodyAttrUrl = 'https://aip.baidubce.com/rest/2.0/image-classify/v1/body_attr'

    __bodyNumUrl = 'https://aip.baidubce.com/rest/2.0/image-classify/v1/body_num'

    
    def bodyAnalysis(self, image, options=None):
        """
            人体关键点识别
        """
        options = options or {}

        data = {}
        data['image'] = base64.b64encode(image).decode()

        data.update(options)

        return self._request(self.__bodyAnalysisUrl, data)
    
    def bodyAttr(self, image, options=None):
        """
            人体属性识别
        """
        options = options or {}

        data = {}
        data['image'] = base64.b64encode(image).decode()

        data.update(options)

        return self._request(self.__bodyAttrUrl, data)
    
    def bodyNum(self, image, options=None):
        """
            人流量统计
        """
        options = options or {}

        data = {}
        data['image'] = base64.b64encode(image).decode()

        data.update(options)

        return self._request(self.__bodyNumUrl, data)
    


# -*- coding: utf-8 -*-

"""
    AipBase
"""
import hmac
import json
import hashlib
import datetime
import base64
import time
import sys
import requests
requests.packages.urllib3.disable_warnings()


if sys.version_info.major == 2:
    from urllib import urlencode
    from urllib import quote
    from urlparse import urlparse
else:
    from urllib.parse import urlencode
    from urllib.parse import quote
    from urllib.parse import urlparse

class AipBase(object):
    """
        AipBase
    """

    __accessTokenUrl = 'https://aip.baidubce.com/oauth/2.0/token'

    __reportUrl = 'https://aip.baidubce.com/rpc/2.0/feedback/v1/report'

    __scope = 'brain_all_scope'

    def __init__(self, appId, apiKey, secretKey):
        """
            AipBase(appId, apiKey, secretKey)
        """

        self._appId = appId.strip()
        self._apiKey = apiKey.strip()
        self._secretKey = secretKey.strip()
        self._authObj = {}
        self._isCloudUser = None
        self.__client = requests
        self.__connectTimeout = 60.0
        self.__socketTimeout = 60.0
        self._proxies = {}
        self.__version = '2_2_7'

    def getVersion(self):
        """
            version
        """
        return self.__version

    def setConnectionTimeoutInMillis(self, ms):
        """
            setConnectionTimeoutInMillis
        """

        self.__connectTimeout = ms / 1000.0

    def setSocketTimeoutInMillis(self, ms):
        """
            setSocketTimeoutInMillis
        """

        self.__socketTimeout = ms / 1000.0

    def setProxies(self, proxies):
        """
            proxies
        """

        self._proxies = proxies

    def _request(self, url, data, headers=None):
        """
            self._request('', {})
        """
        try:
            result = self._validate(url, data)
            if result != True:
                return result

            authObj = self._auth()
            params = self._getParams(authObj)            

            data = self._proccessRequest(url, params, data, headers)
            headers = self._getAuthHeaders('POST', url, params, headers)
            response = self.__client.post(url, data=data, params=params, 
                            headers=headers, verify=False, timeout=(
                                self.__connectTimeout,
                                self.__socketTimeout,
                            ), proxies=self._proxies
                        )
            obj = self._proccessResult(response.content)

            if not self._isCloudUser and obj.get('error_code', '') == 110:
                authObj = self._auth(True)
                params = self._getParams(authObj)
                response = self.__client.post(url, data=data, params=params, 
                                headers=headers, verify=False, timeout=(
                                    self.__connectTimeout,
                                    self.__socketTimeout,
                                ), proxies=self._proxies
                            )
                obj = self._proccessResult(response.content)
        except (requests.exceptions.ReadTimeout, requests.exceptions.ConnectTimeout) as e:
            return {
                'error_code': 'SDK108',
                'error_msg': 'connection or read data timeout',
            }
 
        return obj

    def _validate(self, url, data):
        """
            validate
        """

        return True

    def _proccessRequest(self, url, params, data, headers):
        """
            参数处理
        """

        params['aipSdk'] = 'python'
        params['aipVersion'] = self.__version 

        return data

    def _proccessResult(self, content):
        """
            formate result
        """

        if sys.version_info.major == 2:
            return json.loads(content) or {}
        else:
            return json.loads(content.decode()) or {}

    def _auth(self, refresh=False):
        """
            api access auth
        """

        #未过期
        if not refresh:
            tm = self._authObj.get('time', 0) + int(self._authObj.get('expires_in', 0)) - 30
            if tm > int(time.time()):
                return self._authObj

        obj = self.__client.get(self.__accessTokenUrl, verify=False, params={
            'grant_type': 'client_credentials',
            'client_id': self._apiKey,
            'client_secret': self._secretKey,
        }, timeout=(
            self.__connectTimeout,
            self.__socketTimeout,
        ), proxies=self._proxies).json()

        self._isCloudUser = not self._isPermission(obj)
        obj['time'] = int(time.time())
        self._authObj = obj

        return obj

    def _isPermission(self, authObj):
        """
            check whether permission
        """

        scopes = authObj.get('scope', '') 

        return self.__scope in scopes.split(' ')

    def _getParams(self, authObj):
        """
            api request http url params
        """

        params = {}

        if self._isCloudUser == False:
            params['access_token'] = authObj['access_token']

        return params

    def _getAuthHeaders(self, method, url, params=None, headers=None):
        """
            api request http headers
        """

        headers = headers or {}
        params = params or {}

        if self._isCloudUser == False:
            return headers

        urlResult = urlparse(url)
        for kv in urlResult.query.strip().split('&'):
            if kv:
                k, v = kv.split('=')
                params[k] = v

        # UTC timestamp
        timestamp = datetime.datetime.utcnow().strftime('%Y-%m-%dT%H:%M:%SZ')
        headers['Host'] = urlResult.hostname
        headers['x-bce-date'] = timestamp
        version, expire = '1', '1800'

        # 1 Generate SigningKey
        val = "bce-auth-v%s/%s/%s/%s" % (version, self._apiKey, timestamp, expire)
        signingKey = hmac.new(self._secretKey.encode('utf-8'), val.encode('utf-8'),
            hashlib.sha256
        ).hexdigest()

        # 2 Generate CanonicalRequest
        # 2.1 Genrate CanonicalURI
        canonicalUri = quote(urlResult.path)
        # 2.2 Generate CanonicalURI: not used here
        # 2.3 Generate CanonicalHeaders: only include host here
        
        canonicalHeaders = []
        for header, val in headers.items():
            canonicalHeaders.append(
                '%s:%s' % (
                    quote(header.strip(), '').lower(), 
                    quote(val.strip(), '')
                )
            )
        canonicalHeaders = '\n'.join(sorted(canonicalHeaders))

        # 2.4 Generate CanonicalRequest
        canonicalRequest = '%s\n%s\n%s\n%s' % (
            method.upper(),
            canonicalUri,
            '&'.join(sorted(urlencode(params).split('&'))),
            canonicalHeaders
        )

        # 3 Generate Final Signature 
        signature = hmac.new(signingKey.encode('utf-8'), canonicalRequest.encode('utf-8'),
                        hashlib.sha256
                    ).hexdigest()

        headers['authorization'] = 'bce-auth-v%s/%s/%s/%s/%s/%s' % (
            version,
            self._apiKey,
            timestamp,
            expire, 
            ';'.join(headers.keys()).lower(),
            signature
        )

        return headers

    def report(self, feedback):
        """
            数据反馈
        """

        data = {}
        data['feedback'] = feedback

        return self._request(self.__reportUrl, data)

    def post(self, url, data, headers=None):
        """
            self.post('', {})
        """

        return self._request(url, data, headers)




# -*- coding: utf-8 -*-

"""
知识图谱
"""

import re
import sys
import math
import time
from .base import AipBase
from .base import base64
from .base import json
from .base import urlencode
from .base import quote

class AipKg(AipBase):

    """
    知识图谱
    """

    __createTaskUrl = 'https://aip.baidubce.com/rest/2.0/kg/v1/pie/task_create'

    __updateTaskUrl = 'https://aip.baidubce.com/rest/2.0/kg/v1/pie/task_update'

    __taskInfoUrl = 'https://aip.baidubce.com/rest/2.0/kg/v1/pie/task_info'

    __taskQueryUrl = 'https://aip.baidubce.com/rest/2.0/kg/v1/pie/task_query'

    __taskStartUrl = 'https://aip.baidubce.com/rest/2.0/kg/v1/pie/task_start'

    __taskStatusUrl = 'https://aip.baidubce.com/rest/2.0/kg/v1/pie/task_status'

    
    def createTask(self, name, template_content, input_mapping_file, output_file, url_pattern, options=None):
        """
            创建任务
        """
        options = options or {}

        data = {}
        data['name'] = name
        data['template_content'] = template_content
        data['input_mapping_file'] = input_mapping_file
        data['output_file'] = output_file
        data['url_pattern'] = url_pattern

        data.update(options)

        return self._request(self.__createTaskUrl, data)
    
    def updateTask(self, id, options=None):
        """
            更新任务
        """
        options = options or {}

        data = {}
        data['id'] = id

        data.update(options)

        return self._request(self.__updateTaskUrl, data)
    
    def getTaskInfo(self, id, options=None):
        """
            获取任务详情
        """
        options = options or {}

        data = {}
        data['id'] = id

        data.update(options)

        return self._request(self.__taskInfoUrl, data)
    
    def getUserTasks(self, options=None):
        """
            以分页的方式查询当前用户所有的任务信息
        """
        options = options or {}

        data = {}

        data.update(options)

        return self._request(self.__taskQueryUrl, data)
    
    def startTask(self, id, options=None):
        """
            启动任务
        """
        options = options or {}

        data = {}
        data['id'] = id

        data.update(options)

        return self._request(self.__taskStartUrl, data)
    
    def getTaskStatus(self, id, options=None):
        """
            查询任务状态
        """
        options = options or {}

        data = {}
        data['id'] = id

        data.update(options)

        return self._request(self.__taskStatusUrl, data)
    



# -*- coding: utf-8 -*-

"""
人脸识别
"""

import re
import sys
import math
import time
from .base import AipBase
from .base import base64
from .base import json
from .base import urlencode
from .base import quote

class AipFace(AipBase):

    """
    人脸识别
    """

    __detectUrl = 'https://aip.baidubce.com/rest/2.0/face/v3/detect'

    __searchUrl = 'https://aip.baidubce.com/rest/2.0/face/v3/search'

    __userAddUrl = 'https://aip.baidubce.com/rest/2.0/face/v3/faceset/user/add'

    __userUpdateUrl = 'https://aip.baidubce.com/rest/2.0/face/v3/faceset/user/update'

    __faceDeleteUrl = 'https://aip.baidubce.com/rest/2.0/face/v3/faceset/face/delete'

    __userGetUrl = 'https://aip.baidubce.com/rest/2.0/face/v3/faceset/user/get'

    __faceGetlistUrl = 'https://aip.baidubce.com/rest/2.0/face/v3/faceset/face/getlist'

    __groupGetusersUrl = 'https://aip.baidubce.com/rest/2.0/face/v3/faceset/group/getusers'

    __userCopyUrl = 'https://aip.baidubce.com/rest/2.0/face/v3/faceset/user/copy'

    __userDeleteUrl = 'https://aip.baidubce.com/rest/2.0/face/v3/faceset/user/delete'

    __groupAddUrl = 'https://aip.baidubce.com/rest/2.0/face/v3/faceset/group/add'

    __groupDeleteUrl = 'https://aip.baidubce.com/rest/2.0/face/v3/faceset/group/delete'

    __groupGetlistUrl = 'https://aip.baidubce.com/rest/2.0/face/v3/faceset/group/getlist'

    __personVerifyUrl = 'https://aip.baidubce.com/rest/2.0/face/v3/person/verify'

    __videoSessioncodeUrl = 'https://aip.baidubce.com/rest/2.0/face/v1/faceliveness/sessioncode'

    
    def detect(self, image, image_type, options=None):
        """
            人脸检测
        """
        options = options or {}

        data = {}
        data['image'] = image
        data['image_type'] = image_type

        data.update(options)
        return self._request(self.__detectUrl, json.dumps(data, ensure_ascii=False), {
            'Content-Type': 'application/json',
        })
    
    def search(self, image, image_type, group_id_list, options=None):
        """
            人脸搜索
        """
        options = options or {}

        data = {}
        data['image'] = image
        data['image_type'] = image_type
        data['group_id_list'] = group_id_list

        data.update(options)
        return self._request(self.__searchUrl, json.dumps(data, ensure_ascii=False), {
            'Content-Type': 'application/json',
        })
    
    def addUser(self, image, image_type, group_id, user_id, options=None):
        """
            人脸注册
        """
        options = options or {}

        data = {}
        data['image'] = image
        data['image_type'] = image_type
        data['group_id'] = group_id
        data['user_id'] = user_id

        data.update(options)
        return self._request(self.__userAddUrl, json.dumps(data, ensure_ascii=False), {
            'Content-Type': 'application/json',
        })
    
    def updateUser(self, image, image_type, group_id, user_id, options=None):
        """
            人脸更新
        """
        options = options or {}

        data = {}
        data['image'] = image
        data['image_type'] = image_type
        data['group_id'] = group_id
        data['user_id'] = user_id

        data.update(options)
        return self._request(self.__userUpdateUrl, json.dumps(data, ensure_ascii=False), {
            'Content-Type': 'application/json',
        })
    
    def faceDelete(self, user_id, group_id, face_token, options=None):
        """
            人脸删除
        """
        options = options or {}

        data = {}
        data['user_id'] = user_id
        data['group_id'] = group_id
        data['face_token'] = face_token

        data.update(options)
        return self._request(self.__faceDeleteUrl, json.dumps(data, ensure_ascii=False), {
            'Content-Type': 'application/json',
        })
    
    def getUser(self, user_id, group_id, options=None):
        """
            用户信息查询
        """
        options = options or {}

        data = {}
        data['user_id'] = user_id
        data['group_id'] = group_id

        data.update(options)
        return self._request(self.__userGetUrl, json.dumps(data, ensure_ascii=False), {
            'Content-Type': 'application/json',
        })
    
    def faceGetlist(self, user_id, group_id, options=None):
        """
            获取用户人脸列表
        """
        options = options or {}

        data = {}
        data['user_id'] = user_id
        data['group_id'] = group_id

        data.update(options)
        return self._request(self.__faceGetlistUrl, json.dumps(data, ensure_ascii=False), {
            'Content-Type': 'application/json',
        })
    
    def getGroupUsers(self, group_id, options=None):
        """
            获取用户列表
        """
        options = options or {}

        data = {}
        data['group_id'] = group_id

        data.update(options)
        return self._request(self.__groupGetusersUrl, json.dumps(data, ensure_ascii=False), {
            'Content-Type': 'application/json',
        })
    
    def userCopy(self, user_id, options=None):
        """
            复制用户
        """
        options = options or {}

        data = {}
        data['user_id'] = user_id

        data.update(options)
        return self._request(self.__userCopyUrl, json.dumps(data, ensure_ascii=False), {
            'Content-Type': 'application/json',
        })
    
    def deleteUser(self, group_id, user_id, options=None):
        """
            删除用户
        """
        options = options or {}

        data = {}
        data['group_id'] = group_id
        data['user_id'] = user_id

        data.update(options)
        return self._request(self.__userDeleteUrl, json.dumps(data, ensure_ascii=False), {
            'Content-Type': 'application/json',
        })
    
    def groupAdd(self, group_id, options=None):
        """
            创建用户组
        """
        options = options or {}

        data = {}
        data['group_id'] = group_id

        data.update(options)
        return self._request(self.__groupAddUrl, json.dumps(data, ensure_ascii=False), {
            'Content-Type': 'application/json',
        })
    
    def groupDelete(self, group_id, options=None):
        """
            删除用户组
        """
        options = options or {}

        data = {}
        data['group_id'] = group_id

        data.update(options)
        return self._request(self.__groupDeleteUrl, json.dumps(data, ensure_ascii=False), {
            'Content-Type': 'application/json',
        })
    
    def getGroupList(self, options=None):
        """
            组列表查询
        """
        options = options or {}

        data = {}

        data.update(options)
        return self._request(self.__groupGetlistUrl, json.dumps(data, ensure_ascii=False), {
            'Content-Type': 'application/json',
        })
    
    def personVerify(self, image, image_type, id_card_number, name, options=None):
        """
            身份验证
        """
        options = options or {}

        data = {}
        data['image'] = image
        data['image_type'] = image_type
        data['id_card_number'] = id_card_number
        data['name'] = name

        data.update(options)
        return self._request(self.__personVerifyUrl, json.dumps(data, ensure_ascii=False), {
            'Content-Type': 'application/json',
        })
    
    def videoSessioncode(self, options=None):
        """
            语音校验码接口
        """
        options = options or {}

        data = {}

        data.update(options)
        return self._request(self.__videoSessioncodeUrl, json.dumps(data, ensure_ascii=False), {
            'Content-Type': 'application/json',
        })
    

    __faceverifyUrl = 'https://aip.baidubce.com/rest/2.0/face/v3/faceverify'

    def faceverify(self, images):
        """
            在线活体检测
        """

        return self._request(self.__faceverifyUrl, json.dumps(images, ensure_ascii=False), {
            'Content-Type': 'application/json',
        })

    __matchUrl = 'https://aip.baidubce.com/rest/2.0/face/v3/match'

    def match(self, images):
        """
            人脸比对
        """

        return self._request(self.__matchUrl, json.dumps(images, ensure_ascii=False), {
            'Content-Type': 'application/json',
        })


# -*- coding: utf-8 -*-

"""
    Speech
"""

from .base import AipBase
from .base import base64
from .base import hashlib
from .base import json

class AipSpeech(AipBase):
    """
        Aip Speech
    """

    __asrUrl = 'http://vop.baidu.com/server_api'

    __ttsUrl = 'http://tsn.baidu.com/text2audio'

    def _isPermission(self, authObj):
        """
            check whether permission
        """

        return True    

    def _proccessRequest(self, url, params, data, headers):
        """
            参数处理
        """

        token = params.get('access_token', '')

        if not data.get('cuid', ''):
            data['cuid'] = hashlib.md5(token.encode()).hexdigest()

        if url == self.__asrUrl:
            data['token'] = token
            data = json.dumps(data)
        else:
            data['tok'] = token

        if 'access_token' in params:
            del params['access_token']

        return data

    def _proccessResult(self, content):
        """
            formate result
        """

        try:
            return super(AipSpeech, self)._proccessResult(content)
        except Exception as e:
            return {
                '__json_decode_error': content,
            }

    def asr(self, speech=None, format='pcm', rate=16000, options=None):
        """
            语音识别
        """

        data = {}

        if speech:
            data['speech'] = base64.b64encode(speech).decode()
            data['len'] = len(speech)

        data['channel'] = 1
        data['format'] = format
        data['rate'] = rate

        data = dict(data, **(options or {}))

        return self._request(self.__asrUrl, data)

    def synthesis(self, text, lang='zh', ctp=1, options=None):
        """
            语音合成
        """
        data ={}

        data['tex'] = text
        data['lan'] = lang
        data['ctp'] = ctp

        data = dict(data, **(options or {}))

        result = self._request(self.__ttsUrl, data)

        if '__json_decode_error' in result:
            return result['__json_decode_error']

        return result







# -*- coding: utf-8 -*-

"""
图像识别
"""

import re
import sys
import math
import time
from .base import AipBase
from .base import base64
from .base import json
from .base import urlencode
from .base import quote

class AipOcr(AipBase):

    """
    图像识别
    """

    __generalBasicUrl = 'https://aip.baidubce.com/rest/2.0/ocr/v1/general_basic'

    __accurateBasicUrl = 'https://aip.baidubce.com/rest/2.0/ocr/v1/accurate_basic'

    __generalUrl = 'https://aip.baidubce.com/rest/2.0/ocr/v1/general'

    __accurateUrl = 'https://aip.baidubce.com/rest/2.0/ocr/v1/accurate'

    __generalEnhancedUrl = 'https://aip.baidubce.com/rest/2.0/ocr/v1/general_enhanced'

    __webImageUrl = 'https://aip.baidubce.com/rest/2.0/ocr/v1/webimage'

    __idcardUrl = 'https://aip.baidubce.com/rest/2.0/ocr/v1/idcard'

    __bankcardUrl = 'https://aip.baidubce.com/rest/2.0/ocr/v1/bankcard'

    __drivingLicenseUrl = 'https://aip.baidubce.com/rest/2.0/ocr/v1/driving_license'

    __vehicleLicenseUrl = 'https://aip.baidubce.com/rest/2.0/ocr/v1/vehicle_license'

    __licensePlateUrl = 'https://aip.baidubce.com/rest/2.0/ocr/v1/license_plate'

    __businessLicenseUrl = 'https://aip.baidubce.com/rest/2.0/ocr/v1/business_license'

    __receiptUrl = 'https://aip.baidubce.com/rest/2.0/ocr/v1/receipt'

    __formUrl = 'https://aip.baidubce.com/rest/2.0/ocr/v1/form'

    __tableRecognizeUrl = 'https://aip.baidubce.com/rest/2.0/solution/v1/form_ocr/request'

    __tableResultGetUrl = 'https://aip.baidubce.com/rest/2.0/solution/v1/form_ocr/get_request_result'

    __vatInvoiceUrl = 'https://aip.baidubce.com/rest/2.0/ocr/v1/vat_invoice'

    __qrcodeUrl = 'https://aip.baidubce.com/rest/2.0/ocr/v1/qrcode'

    __numbersUrl = 'https://aip.baidubce.com/rest/2.0/ocr/v1/numbers'

    __lotteryUrl = 'https://aip.baidubce.com/rest/2.0/ocr/v1/lottery'

    __passportUrl = 'https://aip.baidubce.com/rest/2.0/ocr/v1/passport'

    __businessCardUrl = 'https://aip.baidubce.com/rest/2.0/ocr/v1/business_card'

    __handwritingUrl = 'https://aip.baidubce.com/rest/2.0/ocr/v1/handwriting'

    __customUrl = 'https://aip.baidubce.com/rest/2.0/solution/v1/iocr/recognise'

    
    def basicGeneral(self, image, options=None):
        """
            通用文字识别
        """
        options = options or {}

        data = {}
        data['image'] = base64.b64encode(image).decode()

        data.update(options)

        return self._request(self.__generalBasicUrl, data)
    
    def basicGeneralUrl(self, url, options=None):
        """
            通用文字识别
        """
        options = options or {}

        data = {}
        data['url'] = url

        data.update(options)

        return self._request(self.__generalBasicUrl, data)
    
    def basicAccurate(self, image, options=None):
        """
            通用文字识别（高精度版）
        """
        options = options or {}

        data = {}
        data['image'] = base64.b64encode(image).decode()

        data.update(options)

        return self._request(self.__accurateBasicUrl, data)
    
    def general(self, image, options=None):
        """
            通用文字识别（含位置信息版）
        """
        options = options or {}

        data = {}
        data['image'] = base64.b64encode(image).decode()

        data.update(options)

        return self._request(self.__generalUrl, data)
    
    def generalUrl(self, url, options=None):
        """
            通用文字识别（含位置信息版）
        """
        options = options or {}

        data = {}
        data['url'] = url

        data.update(options)

        return self._request(self.__generalUrl, data)
    
    def accurate(self, image, options=None):
        """
            通用文字识别（含位置高精度版）
        """
        options = options or {}

        data = {}
        data['image'] = base64.b64encode(image).decode()

        data.update(options)

        return self._request(self.__accurateUrl, data)
    
    def enhancedGeneral(self, image, options=None):
        """
            通用文字识别（含生僻字版）
        """
        options = options or {}

        data = {}
        data['image'] = base64.b64encode(image).decode()

        data.update(options)

        return self._request(self.__generalEnhancedUrl, data)
    
    def enhancedGeneralUrl(self, url, options=None):
        """
            通用文字识别（含生僻字版）
        """
        options = options or {}

        data = {}
        data['url'] = url

        data.update(options)

        return self._request(self.__generalEnhancedUrl, data)
    
    def webImage(self, image, options=None):
        """
            网络图片文字识别
        """
        options = options or {}

        data = {}
        data['image'] = base64.b64encode(image).decode()

        data.update(options)

        return self._request(self.__webImageUrl, data)
    
    def webImageUrl(self, url, options=None):
        """
            网络图片文字识别
        """
        options = options or {}

        data = {}
        data['url'] = url

        data.update(options)

        return self._request(self.__webImageUrl, data)
    
    def idcard(self, image, id_card_side, options=None):
        """
            身份证识别
        """
        options = options or {}

        data = {}
        data['image'] = base64.b64encode(image).decode()
        data['id_card_side'] = id_card_side

        data.update(options)

        return self._request(self.__idcardUrl, data)
    
    def bankcard(self, image, options=None):
        """
            银行卡识别
        """
        options = options or {}

        data = {}
        data['image'] = base64.b64encode(image).decode()

        data.update(options)

        return self._request(self.__bankcardUrl, data)
    
    def drivingLicense(self, image, options=None):
        """
            驾驶证识别
        """
        options = options or {}

        data = {}
        data['image'] = base64.b64encode(image).decode()

        data.update(options)

        return self._request(self.__drivingLicenseUrl, data)
    
    def vehicleLicense(self, image, options=None):
        """
            行驶证识别
        """
        options = options or {}

        data = {}
        data['image'] = base64.b64encode(image).decode()

        data.update(options)

        return self._request(self.__vehicleLicenseUrl, data)
    
    def licensePlate(self, image, options=None):
        """
            车牌识别
        """
        options = options or {}

        data = {}
        data['image'] = base64.b64encode(image).decode()

        data.update(options)

        return self._request(self.__licensePlateUrl, data)
    
    def businessLicense(self, image, options=None):
        """
            营业执照识别
        """
        options = options or {}

        data = {}
        data['image'] = base64.b64encode(image).decode()

        data.update(options)

        return self._request(self.__businessLicenseUrl, data)
    
    def receipt(self, image, options=None):
        """
            通用票据识别
        """
        options = options or {}

        data = {}
        data['image'] = base64.b64encode(image).decode()

        data.update(options)

        return self._request(self.__receiptUrl, data)
    
    def form(self, image, options=None):
        """
            表格文字识别同步接口
        """
        options = options or {}

        data = {}
        data['image'] = base64.b64encode(image).decode()

        data.update(options)

        return self._request(self.__formUrl, data)
    
    def tableRecognitionAsync(self, image, options=None):
        """
            表格文字识别
        """
        options = options or {}

        data = {}
        data['image'] = base64.b64encode(image).decode()

        data.update(options)

        return self._request(self.__tableRecognizeUrl, data)
    
    def getTableRecognitionResult(self, request_id, options=None):
        """
            表格识别结果
        """
        options = options or {}

        data = {}
        data['request_id'] = request_id

        data.update(options)

        return self._request(self.__tableResultGetUrl, data)
    
    def vatInvoice(self, image, options=None):
        """
            增值税发票识别
        """
        options = options or {}

        data = {}
        data['image'] = base64.b64encode(image).decode()

        data.update(options)

        return self._request(self.__vatInvoiceUrl, data)
    
    def qrcode(self, image, options=None):
        """
            二维码识别
        """
        options = options or {}

        data = {}
        data['image'] = base64.b64encode(image).decode()

        data.update(options)

        return self._request(self.__qrcodeUrl, data)
    
    def numbers(self, image, options=None):
        """
            数字识别
        """
        options = options or {}

        data = {}
        data['image'] = base64.b64encode(image).decode()

        data.update(options)

        return self._request(self.__numbersUrl, data)
    
    def lottery(self, image, options=None):
        """
            彩票识别
        """
        options = options or {}

        data = {}
        data['image'] = base64.b64encode(image).decode()

        data.update(options)

        return self._request(self.__lotteryUrl, data)
    
    def passport(self, image, options=None):
        """
            护照识别
        """
        options = options or {}

        data = {}
        data['image'] = base64.b64encode(image).decode()

        data.update(options)

        return self._request(self.__passportUrl, data)
    
    def businessCard(self, image, options=None):
        """
            名片识别
        """
        options = options or {}

        data = {}
        data['image'] = base64.b64encode(image).decode()

        data.update(options)

        return self._request(self.__businessCardUrl, data)
    
    def handwriting(self, image, options=None):
        """
            手写文字识别
        """
        options = options or {}

        data = {}
        data['image'] = base64.b64encode(image).decode()

        data.update(options)

        return self._request(self.__handwritingUrl, data)
    
    def custom(self, image, templateSign, options=None):
        """
            自定义模板文字识别
        """
        options = options or {}

        data = {}
        data['image'] = base64.b64encode(image).decode()
        data['templateSign'] = templateSign

        data.update(options)

        return self._request(self.__customUrl, data)
    
    def tableRecognition(self, image, options=None, timeout=10000):
        """
            tableRecognition
        """
        
        result = self.tableRecognitionAsync(image)

        if 'error_code' in result:
            return result
        
        requestId = result['result'][0]['request_id']
        for i in range(int(math.ceil(timeout / 1000.0))):
            result = self.getTableRecognitionResult(requestId, options)
            
            # 完成
            if int(result['result']['ret_code']) == 3: 
                break
            time.sleep(1)

        return result



# -*- coding: utf-8 -*-

"""
图像识别
"""

import re
import sys
import math
import time
from .base import AipBase
from .base import base64
from .base import json
from .base import urlencode
from .base import quote

class AipImageClassify(AipBase):

    """
    图像识别
    """

    __advancedGeneralUrl = 'https://aip.baidubce.com/rest/2.0/image-classify/v2/advanced_general'

    __dishDetectUrl = 'https://aip.baidubce.com/rest/2.0/image-classify/v2/dish'

    __carDetectUrl = 'https://aip.baidubce.com/rest/2.0/image-classify/v1/car'

    __logoSearchUrl = 'https://aip.baidubce.com/rest/2.0/image-classify/v2/logo'

    __logoAddUrl = 'https://aip.baidubce.com/rest/2.0/realtime_search/v1/logo/add'

    __logoDeleteUrl = 'https://aip.baidubce.com/rest/2.0/realtime_search/v1/logo/delete'

    __animalDetectUrl = 'https://aip.baidubce.com/rest/2.0/image-classify/v1/animal'

    __plantDetectUrl = 'https://aip.baidubce.com/rest/2.0/image-classify/v1/plant'

    __objectDetectUrl = 'https://aip.baidubce.com/rest/2.0/image-classify/v1/object_detect'

    
    def advancedGeneral(self, image, options=None):
        """
            通用物体识别
        """
        options = options or {}

        data = {}
        data['image'] = base64.b64encode(image).decode()

        data.update(options)

        return self._request(self.__advancedGeneralUrl, data)
    
    def dishDetect(self, image, options=None):
        """
            菜品识别
        """
        options = options or {}

        data = {}
        data['image'] = base64.b64encode(image).decode()

        data.update(options)

        return self._request(self.__dishDetectUrl, data)
    
    def carDetect(self, image, options=None):
        """
            车辆识别
        """
        options = options or {}

        data = {}
        data['image'] = base64.b64encode(image).decode()

        data.update(options)

        return self._request(self.__carDetectUrl, data)
    
    def logoSearch(self, image, options=None):
        """
            logo商标识别
        """
        options = options or {}

        data = {}
        data['image'] = base64.b64encode(image).decode()

        data.update(options)

        return self._request(self.__logoSearchUrl, data)
    
    def logoAdd(self, image, brief, options=None):
        """
            logo商标识别—添加
        """
        options = options or {}

        data = {}
        data['image'] = base64.b64encode(image).decode()
        data['brief'] = brief

        data.update(options)

        return self._request(self.__logoAddUrl, data)
    
    def logoDeleteByImage(self, image, options=None):
        """
            logo商标识别—删除
        """
        options = options or {}

        data = {}
        data['image'] = base64.b64encode(image).decode()

        data.update(options)

        return self._request(self.__logoDeleteUrl, data)
    
    def logoDeleteBySign(self, cont_sign, options=None):
        """
            logo商标识别—删除
        """
        options = options or {}

        data = {}
        data['cont_sign'] = cont_sign

        data.update(options)

        return self._request(self.__logoDeleteUrl, data)
    
    def animalDetect(self, image, options=None):
        """
            动物识别
        """
        options = options or {}

        data = {}
        data['image'] = base64.b64encode(image).decode()

        data.update(options)

        return self._request(self.__animalDetectUrl, data)
    
    def plantDetect(self, image, options=None):
        """
            植物识别
        """
        options = options or {}

        data = {}
        data['image'] = base64.b64encode(image).decode()

        data.update(options)

        return self._request(self.__plantDetectUrl, data)
    
    def objectDetect(self, image, options=None):
        """
            图像主体检测
        """
        options = options or {}

        data = {}
        data['image'] = base64.b64encode(image).decode()

        data.update(options)

        return self._request(self.__objectDetectUrl, data)
    